#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include "clsPCA9555.h"
#include "Wire.h"


#include <Adafruit_NeoPixel.h>
#include "SensorLock.h"

PCA9555 I2CExpander(0x20);
Adafruit_NeoPixel LEDS = Adafruit_NeoPixel(PIXEL_NUM, LED_PIN, NEO_GRB + NEO_KHZ800);


const uint8_t I2CExpLockPins[LOCK_PINS_COUNT] = {LOCK_PIN_0, LOCK_PIN_1, LOCK_PIN_2, LOCK_PIN_3,
                                           LOCK_PIN_4, LOCK_PIN_5, LOCK_PIN_6, LOCK_PIN_7,};

const uint8_t PASSWORD[PASSWORD_LENGTH] = {(1 << 0), (1 << 1), (1 << 2), (1 << 3),
                                           (1 << 4), (1 << 5), (1 << 6), (1 << 7),};

void setup()
{
  pinMode(BUZZER_PIN, OUTPUT);
  pinMode(RELAY_PIN, OUTPUT);
  
  LEDS.begin();
  for (uint8_t LEDNum = 0; LEDNum < PIXEL_NUM; LEDNum++)
    LEDS.setPixelColor(LEDNum, COLOR_LOCKED);
  LEDS.show();
}

void loop()
{
  uint8_t ReadVal = ReadLockPins();

  if (ReadVal != 0)
  {
    bool OneSensorActive = false;
    bool UserError = true;

    delay(150);
    ReadVal = ReadLockPins();
        
    for (uint8_t pin = 0; pin < LOCK_PINS_COUNT; pin++)
    {
      if (ReadVal == (1 << pin))
      {
        OneSensorActive = true;
        break;
      }
    }

    if (OneSensorActive) //ввод пароля
    {
      for (uint8_t LEDNum = 0; LEDNum < PIXEL_NUM; LEDNum++)
        LEDS.setPixelColor(LEDNum, COLOR_INPUT);
      LEDS.show();

      analogWrite(BUZZER_PIN, 128);
      delay(BuzzerDelay);
      analogWrite(BUZZER_PIN, 0);
      
      uint8_t UserPassword[PASSWORD_LENGTH];
      UserPassword[0] = ReadVal;
      UserError = InputErrorCheck();
      
      for (uint8_t cycle = 1; cycle < PASSWORD_LENGTH; cycle++)
      {
        if (UserError)
          break;
        
        UserError = NewData(&UserPassword[cycle]);
        if (UserError)
          break;

        analogWrite(BUZZER_PIN, 128);
        delay(BuzzerDelay);
        analogWrite(BUZZER_PIN, 0);
      
        UserError = InputErrorCheck();   
      }

      if (!UserError)
      {
        bool PasswordError = false;
        
        for (uint8_t i = 0; i < PASSWORD_LENGTH; i++)
        {
          if (PASSWORD[i] != UserPassword[i])
          {
            PasswordError = true;
            break;
          }
        }

        if (!PasswordError)
        {
          digitalWrite(RELAY_PIN, HIGH);
          for (uint8_t LEDNum = (PIXEL_NUM - 1); LEDNum >= 1; LEDNum--)
          {
            LEDS.setPixelColor(LEDNum, COLOR_UNLOCKED);
            LEDS.show();
            analogWrite(BUZZER_PIN, LEDNum * 36);
            delay(OneLEDDelay);
          }
          LEDS.setPixelColor(0, COLOR_UNLOCKED);
          LEDS.show();
          analogWrite(BUZZER_PIN, 0);
        }

        else
        {
          UserError = true;
        }
      }
    }
    
    else if (ReadVal != 0) //ввод команды
    {
      if (ReadVal == CMD_LOCK)
      {
        UserError = false;
        digitalWrite(RELAY_PIN, LOW);
        for (uint8_t LEDNum = 0; LEDNum < PIXEL_NUM; LEDNum++)
        {
          LEDS.setPixelColor(LEDNum, COLOR_LOCKED);
          LEDS.show();
          analogWrite(BUZZER_PIN, LEDNum * 36);
          delay(OneLEDDelay);
        }
        //LEDS.show();
        analogWrite(BUZZER_PIN, 32);
        delay(OneLEDDelay);
        analogWrite(BUZZER_PIN, 0);
      }

      else
        UserError = true;
    }

    else
    {
      UserError = false;
      
      uint32_t ColorToSet = COLOR_LOCKED;
      if (digitalRead(RELAY_PIN) == HIGH)
        ColorToSet = COLOR_UNLOCKED;

      for (uint8_t LEDNum = 0; LEDNum < PIXEL_NUM; LEDNum++)
        LEDS.setPixelColor(LEDNum, ColorToSet);
      LEDS.show();
    }

    if (UserError)
    {
      for (uint8_t LEDNum = 0; LEDNum < PIXEL_NUM; LEDNum++)
        LEDS.setPixelColor(LEDNum, COLOR_PAUSE);
      LEDS.show();

      for (uint8_t i = 0; i < 2; i++)
      {
        analogWrite(BUZZER_PIN, 128);
        delay(BuzzerDelay);
        analogWrite(BUZZER_PIN, 0);
        delay(BuzzerDelay);
      }
    }

    delay(DelayBetweenInput);

    uint32_t ColorToSet = COLOR_LOCKED;
    if (digitalRead(RELAY_PIN) == HIGH)
    ColorToSet = COLOR_UNLOCKED;
    
    if (UserError)
    {
      for (uint8_t LEDNum = 0; LEDNum < PIXEL_NUM; LEDNum++)
        LEDS.setPixelColor(LEDNum, ColorToSet);
      LEDS.show();
    }

    else
    {
      for (uint8_t LEDNum = 0; LEDNum < PIXEL_NUM; LEDNum++)
        LEDS.setPixelColor(LEDNum, COLOR_PAUSE);
      LEDS.show();

      delay(500);
      
      for (uint8_t LEDNum = 0; LEDNum < PIXEL_NUM; LEDNum++)
        LEDS.setPixelColor(LEDNum, ColorToSet);
      LEDS.show();
    }
  }
}

//---------------------------------------------------
uint8_t ReadLockPins()
{
  uint8_t PinsState = 0;
  
  for (uint8_t pin = 0; pin < LOCK_PINS_COUNT; pin++)
    PinsState |= I2CExpander.digitalRead(I2CExpLockPins[pin]) << pin;

  return PinsState;
}
//---------------------------------------------------
bool InputErrorCheck()
{
  bool Error = true;

  for (uint8_t DelayTime = 0; DelayTime < DelayTimesMax; DelayTime++)
  {
    delay(OneTimeDelay);
    if (ReadLockPins() == 0)
    {
      Error = false;
      break;
    }
  }

  return Error;
}
//---------------------------------------------------
bool NewData(uint8_t *Data)
{
  bool Error = true;

  for (uint8_t DelayTime = 0; DelayTime < DelayTimesMax; DelayTime++)
  {
    delay(OneTimeDelay);
    *Data = ReadLockPins();
    if (*Data != 0)
    {
      Error = false;
      break;
    }
  }

  return Error;
}




